require 'nodesets'

# TODO: Add further tests
#       Fix the :human_sort, :human_sort! test
describe Nodesets do
  it "Nodeset pre-ordered with spaces before or after in names contains ' iam1','iam9 ','iam10',' iam11 ',' iam20'" do
    arr = ['iam1','iam9','iam10','iam11','iam20']
    expect(Nodesets::Nodeset.new(arr.join(',')).to_a).to eql(arr)
  end

  it "Nodeset pre-ordered contains iam1,iam9,iam10,iam11,iam20" do
    arr = ['iam1','iam9','iam10','iam11','iam20']
    expect(Nodesets::Nodeset.new(arr.join(',')).to_a).to eql(arr)
  end

  it "Nodeset pre-unordered contains iam9,iam10,iam11,iam20,iam1" do
    arr = ['iam9','iam10','iam11','iam20','iam1']
    expect(Nodesets::Nodeset.new(arr.join(',')).to_a).to eql(arr)
  end

  it "Nodeset human_sorted" do
    unsorted_arr = ['iam9','iam10','iam11','iam20','iam1']
    sorted_arr = ['iam1','iam9','iam10','iam11','iam20']
    expect(Nodesets::Nodeset.new(unsorted_arr.join(',')).human_sort.to_a).to eql(sorted_arr)
  end

  it "Nodeset human_sort and human_sort! to return the same sorted array when to_a" do
    unsorted_arr = ['iam9','iam10','iam11','iam20','iam1']
    initial = Nodesets::Nodeset.new(unsorted_arr.join(','))
    human_sort_wo_dup = initial.human_sort
    human_sort_w_dup  = initial.human_sort!
    expect(human_sort_wo_dup.to_a == human_sort_w_dup.to_a).to be true
  end

  ### TODO: FIXME: This test currently does not work
  ## The purpose of this test is to verify that :human_sort returns a new object, and 
  ##  :human_sort! replaces the current object with a new object, unsure of how to test this...
  #it "Nodeset human_sort! (duplicate, new, object returned rather than the same object)" do
  #  unsorted_arr = ['iam9','iam10','iam11','iam20','iam1']
  #  initial = Nodesets::Nodeset.new(unsorted_arr.join(','))
  #  human_sort_wo_dup = initial.human_sort
  #  human_sort_w_dup  = initial.human_sort!
  #  expect(human_sort_wo_dup == human_sort_w_dup).to be true
  #  expect(human_sort_wo_dup.object_id == human_sort_w_dup.object_id).to be false
  #end

  it "Nodeset after value merge to contain the same values(not order specific) as array" do
    arr = ['iam1','iam9','iam10','iam11','iam20']
    initial = Nodesets::Nodeset.new(arr.join(','))
    to_adjust = 'iam3'
    adjusted_arr = (arr << to_adjust)
    adjusted_nodeset_arr = initial.merge(to_adjust).to_a
    expect(adjusted_nodeset_arr & adjusted_arr == adjusted_arr && adjusted_nodeset_arr - adjusted_arr == []).to be true
  end

  it "Nodeset after value subtraction to contain the same values(not order specific) as array" do
    arr = ['iam1','iam9','iam10','iam11','iam20']
    initial = Nodesets::Nodeset.new(arr.join(','))
    to_adjust = 'iam11'
    adjusted_arr = arr - [to_adjust]
    adjusted_nodeset_arr = initial.subtract(to_adjust).to_a
    expect(adjusted_nodeset_arr & adjusted_arr == adjusted_arr && adjusted_nodeset_arr - adjusted_arr == []).to be true
  end

  it "Nodeset :& intersection converted back to array to match array intersection(not order specific)" do
    arr = ['iam1','iam9','iam10','iam11','iam20']
    intersection = ['iam9','iam11','iam400']
    initial = Nodesets::Nodeset.new(arr.join(','))
    adjusted_arr = arr & intersection
    adjusted_nodeset_arr = (initial & Nodesets::Nodeset.new(intersection.join(','))).to_a
    expect(adjusted_nodeset_arr & adjusted_arr == adjusted_arr && adjusted_nodeset_arr - adjusted_arr == []).to be true
  end

  it "Nodeset fold properly" do
    arr = ['iam1','iam9','iam10','iam11','iam20']
    expect(Nodesets::Nodeset.new(arr.join(',')).fold).to eql('iam[1,9-11,20]')
  end

  it "Nodeset expand properly and be :human_sort'ed" do
    arr = ['iam13','iam1','iam9','iam10','iam11','iam20']
    human_sorted_arr = ['iam1','iam9','iam10','iam11','iam13','iam20']
    expect(Nodesets::Nodeset.new(arr.join(',')).expand).to eql(human_sorted_arr)
  end

  it "Nodeset XOR properly" do
    arr = ['iam1','iam9','iam10','iam11','iam20']
    xor_arr = ['iam1','iam10','iam11']
    shouldbe_arr = ['iam9','iam20']
    expect((Nodesets::Nodeset.new(arr.join(',')) ^ Nodesets::Nodeset.new(xor_arr.join(','))).to_a).to eql(shouldbe_arr)
  end

  it "Nodeset unsorted array to sorted subset :[] properly" do
    arr = ['iam1','iam15','iam9','iam10','iam11','iam20']
    expect(Nodesets::Nodeset.new(arr.join(','))[2..4]).to eql(['iam10','iam11','iam15'])
  end

end
